@@ -20,7 +20,7 @@
 
 /**
  * SECTION:element-oggmux
- * @see_also: <link linkend="gst-plugins-base-plugins-deoggmux">oggdemux</link>
+ * @see_also: <link linkend="gst-plugins-base-plugins-oggdemux">oggdemux</link>
  *
  * This element merges streams (audio and video) into ogg files.
  *
@@ -41,6 +41,7 @@
 
 #include <gst/gst.h>
 #include <gst/base/gstcollectpads.h>
+#include <gst/tag/tag.h>
 
 #include "gstoggmux.h"
 
@@ -62,6 +63,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_ogg_mux_debug);
     : GST_BUFFER_TIMESTAMP (buf))
 
 #define GST_GP_FORMAT "[gp %8" G_GINT64_FORMAT "]"
+#define GST_GP_CAST(_gp) ((gint64) _gp)
 
 typedef enum
 {
@@ -70,13 +72,6 @@ typedef enum
 }
 GstOggFlag;
 
-/* elementfactory information */
-static const GstElementDetails gst_ogg_mux_details =
-GST_ELEMENT_DETAILS ("Ogg muxer",
-    "Codec/Muxer",
-    "mux ogg streams (info about ogg: http://xiph.org)",
-    "Wim Taymans <wim@fluendo.com>");
-
 /* OggMux signals and args */
 enum
 {
@@ -106,7 +101,7 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink_%d",
     GST_STATIC_CAPS ("video/x-theora; "
         "audio/x-vorbis; audio/x-flac; audio/x-speex; audio/x-celt; "
         "application/x-ogm-video; application/x-ogm-audio; video/x-dirac; "
-        "video/x-smoke; text/x-cmml, encoded = (boolean) TRUE; "
+        "video/x-smoke; video/x-vp8; text/x-cmml, encoded = (boolean) TRUE; "
         "subtitle/x-kate; application/x-kate")
     );
 
@@ -175,7 +170,10 @@ gst_ogg_mux_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_factory));
 
-  gst_element_class_set_details (element_class, &gst_ogg_mux_details);
+  gst_element_class_set_details_simple (element_class,
+      "Ogg muxer", "Codec/Muxer",
+      "mux ogg streams (info about ogg: http://xiph.org)",
+      "Wim Taymans <wim@fluendo.com>");
 }
 
 static void
@@ -230,8 +228,6 @@ gst_ogg_mux_clear (GstOggMux * ogg_mux)
 {
   ogg_mux->pulling = NULL;
   ogg_mux->need_headers = TRUE;
-  ogg_mux->max_delay = DEFAULT_MAX_DELAY;
-  ogg_mux->max_page_delay = DEFAULT_MAX_PAGE_DELAY;
   ogg_mux->delta_pad = NULL;
   ogg_mux->offset = 0;
   ogg_mux->next_ts = 0;
@@ -259,6 +255,9 @@ gst_ogg_mux_init (GstOggMux * ogg_mux)
       (GstCollectPadsFunction) GST_DEBUG_FUNCPTR (gst_ogg_mux_collected),
       ogg_mux);
 
+  ogg_mux->max_delay = DEFAULT_MAX_DELAY;
+  ogg_mux->max_page_delay = DEFAULT_MAX_PAGE_DELAY;
+
   gst_ogg_mux_clear (ogg_mux);
 }
 
@@ -280,7 +279,7 @@ gst_ogg_mux_finalize (GObject * object)
 static void
 gst_ogg_mux_ogg_pad_destroy_notify (GstCollectData * data)
 {
-  GstOggPad *oggpad = (GstOggPad *) data;
+  GstOggPadData *oggpad = (GstOggPadData *) data;
   GstBuffer *buf;
 
   ogg_stream_clear (&oggpad->stream);
@@ -312,7 +311,7 @@ static gboolean
 gst_ogg_mux_sink_event (GstPad * pad, GstEvent * event)
 {
   GstOggMux *ogg_mux = GST_OGG_MUX (gst_pad_get_parent (pad));
-  GstOggPad *ogg_pad = (GstOggPad *) gst_pad_get_element_private (pad);
+  GstOggPadData *ogg_pad = (GstOggPadData *) gst_pad_get_element_private (pad);
   gboolean ret;
 
   GST_DEBUG ("Got %s event on pad %s:%s", GST_EVENT_TYPE_NAME (event),
@@ -378,11 +377,11 @@ gst_ogg_mux_request_new_pad (GstElement * element,
     /* construct our own wrapper data structure for the pad to
      * keep track of its status */
     {
-      GstOggPad *oggpad;
+      GstOggPadData *oggpad;
 
-      oggpad = (GstOggPad *)
+      oggpad = (GstOggPadData *)
           gst_collect_pads_add_pad_full (ogg_mux->collect, newpad,
-          sizeof (GstOggPad), gst_ogg_mux_ogg_pad_destroy_notify);
+          sizeof (GstOggPadData), gst_ogg_mux_ogg_pad_destroy_notify);
       ogg_mux->active_pads++;
 
       oggpad->serial = serial;
@@ -475,7 +474,8 @@ gst_ogg_mux_buffer_from_page (GstOggMux * mux, ogg_page * page, gboolean delta)
     GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT);
 
   GST_LOG_OBJECT (mux, GST_GP_FORMAT
-      " created buffer %p from ogg page", ogg_page_granulepos (page), buffer);
+      " created buffer %p from ogg page",
+      GST_GP_CAST (ogg_page_granulepos (page)), buffer);
 
   return buffer;
 }
@@ -513,7 +513,7 @@ static gboolean
 gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
 {
   GSList *walk;
-  GstOggPad *opad = NULL;       /* "oldest" pad */
+  GstOggPadData *opad = NULL;   /* "oldest" pad */
   GstClockTime oldest = GST_CLOCK_TIME_NONE;
   GstBuffer *buf = NULL;
   gboolean ret = FALSE;
@@ -522,7 +522,7 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
 
   walk = mux->collect->data;
   while (walk) {
-    GstOggPad *pad = (GstOggPad *) walk->data;
+    GstOggPadData *pad = (GstOggPadData *) walk->data;
 
     /* We need each queue to either be at EOS, or have one or more pages
      * available with a set granulepos (i.e. not -1), otherwise we don't have
@@ -563,7 +563,7 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
 
   walk = mux->collect->data;
   while (walk) {
-    GstOggPad *pad = (GstOggPad *) walk->data;
+    GstOggPadData *pad = (GstOggPadData *) walk->data;
 
     /* any page with a granulepos of -1 can be pushed immediately.
      * TODO: it CAN be, but it seems silly to do so? */
@@ -626,8 +626,8 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
  * counting.
  */
 static GstFlowReturn
-gst_ogg_mux_pad_queue_page (GstOggMux * mux, GstOggPad * pad, ogg_page * page,
-    gboolean delta)
+gst_ogg_mux_pad_queue_page (GstOggMux * mux, GstOggPadData * pad,
+    ogg_page * page, gboolean delta)
 {
   GstFlowReturn ret;
   GstBuffer *buffer = gst_ogg_mux_buffer_from_page (mux, page, delta);
@@ -645,7 +645,7 @@ gst_ogg_mux_pad_queue_page (GstOggMux * mux, GstOggPad * pad, ogg_page * page,
   GST_LOG_OBJECT (pad->collect.pad, GST_GP_FORMAT
       " queued buffer page %p (gp time %"
       GST_TIME_FORMAT ", timestamp %" GST_TIME_FORMAT
-      "), %d page buffers queued", ogg_page_granulepos (page),
+      "), %d page buffers queued", GST_GP_CAST (ogg_page_granulepos (page)),
       buffer, GST_TIME_ARGS (GST_BUFFER_OFFSET (buffer)),
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
       g_queue_get_length (pad->pagebuffers));
@@ -668,8 +668,8 @@ gst_ogg_mux_pad_queue_page (GstOggMux * mux, GstOggPad * pad, ogg_page * page,
  * of muxed pages
  */
 static gint
-gst_ogg_mux_compare_pads (GstOggMux * ogg_mux, GstOggPad * first,
-    GstOggPad * second)
+gst_ogg_mux_compare_pads (GstOggMux * ogg_mux, GstOggPadData * first,
+    GstOggPadData * second)
 {
   guint64 firsttime, secondtime;
 
@@ -728,20 +728,20 @@ gst_ogg_mux_compare_pads (GstOggMux * ogg_mux, GstOggPad * first,
  * NULL when no pad was usable. "best" means the buffer marked
  * with the lowest timestamp. If best->buffer == NULL then nothing
  * should be done until more data arrives */
-static GstOggPad *
+static GstOggPadData *
 gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
 {
-  GstOggPad *bestpad = NULL, *still_hungry = NULL;
+  GstOggPadData *bestpad = NULL, *still_hungry = NULL;
   GSList *walk;
 
   /* try to make sure we have a buffer from each usable pad first */
   walk = ogg_mux->collect->data;
   while (walk) {
-    GstOggPad *pad;
+    GstOggPadData *pad;
     GstCollectData *data;
 
     data = (GstCollectData *) walk->data;
-    pad = (GstOggPad *) data;
+    pad = (GstOggPadData *) data;
 
     walk = g_slist_next (walk);
 
@@ -837,7 +837,7 @@ gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
 }
 
 static GList *
-gst_ogg_mux_get_headers (GstOggPad * pad)
+gst_ogg_mux_get_headers (GstOggPadData * pad)
 {
   GList *res = NULL;
   GstStructure *structure;
@@ -878,6 +878,10 @@ gst_ogg_mux_get_headers (GstOggPad * pad)
       } else {
         GST_LOG_OBJECT (thepad, "streamheader is not fixed list");
       }
+
+      /* Start a new page for every CMML buffer */
+      if (gst_structure_has_name (structure, "text/x-cmml"))
+        pad->always_flush_page = TRUE;
     } else if (gst_structure_has_name (structure, "video/x-dirac")) {
       res = g_list_append (res, pad->buffer);
       pad->buffer = pad->next_buffer;
@@ -954,10 +958,10 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
 
   walk = mux->collect->data;
   while (walk) {
-    GstOggPad *pad;
+    GstOggPadData *pad;
     GstPad *thepad;
 
-    pad = (GstOggPad *) walk->data;
+    pad = (GstOggPadData *) walk->data;
     thepad = pad->collect.pad;
 
     walk = g_slist_next (walk);
@@ -975,7 +979,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
   GST_LOG_OBJECT (mux, "creating BOS pages");
   walk = mux->collect->data;
   while (walk) {
-    GstOggPad *pad;
+    GstOggPadData *pad;
     GstBuffer *buf;
     ogg_packet packet;
     ogg_page page;
@@ -984,7 +988,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     GstStructure *structure;
     GstBuffer *hbuf;
 
-    pad = (GstOggPad *) walk->data;
+    pad = (GstOggPadData *) walk->data;
     thepad = pad->collect.pad;
     caps = gst_pad_get_negotiated_caps (thepad);
     structure = gst_caps_get_structure (caps, 0);
@@ -1038,7 +1042,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     GST_LOG_OBJECT (mux, "swapped out page with mime type %s",
         gst_structure_get_name (structure));
 
-    /* quick hack: put Theora and Dirac video pages at the front.
+    /* quick hack: put Theora, VP8 and Dirac video pages at the front.
      * Ideally, we would have a settable enum for which Ogg
      * profile we work with, and order based on that.
      * (FIXME: if there is more than one video stream, shouldn't we only put
@@ -1047,11 +1051,13 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     if (gst_structure_has_name (structure, "video/x-theora")) {
       GST_DEBUG_OBJECT (thepad, "putting %s page at the front", "Theora");
       hbufs = g_list_prepend (hbufs, hbuf);
-      pad->always_flush_page = TRUE;
     } else if (gst_structure_has_name (structure, "video/x-dirac")) {
       GST_DEBUG_OBJECT (thepad, "putting %s page at the front", "Dirac");
       hbufs = g_list_prepend (hbufs, hbuf);
       pad->always_flush_page = TRUE;
+    } else if (gst_structure_has_name (structure, "video/x-vp8")) {
+      GST_DEBUG_OBJECT (thepad, "putting %s page at the front", "VP8");
+      hbufs = g_list_prepend (hbufs, hbuf);
     } else {
       hbufs = g_list_append (hbufs, hbuf);
     }
@@ -1061,10 +1067,10 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
   GST_LOG_OBJECT (mux, "creating next headers");
   walk = mux->collect->data;
   while (walk) {
-    GstOggPad *pad;
+    GstOggPadData *pad;
     GstPad *thepad;
 
-    pad = (GstOggPad *) walk->data;
+    pad = (GstOggPadData *) walk->data;
     thepad = pad->collect.pad;
 
     walk = walk->next;
@@ -1166,7 +1172,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
  *    pads are at EOS)
  */
 static GstFlowReturn
-gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
+gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPadData * best)
 {
   GstFlowReturn ret = GST_FLOW_OK;
   gboolean delta_unit;
@@ -1189,8 +1195,7 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
    * for the pad we were pulling from before */
   if (ogg_mux->pulling && best &&
       ogg_mux->pulling != best && ogg_mux->pulling->buffer) {
-    GstOggPad *pad = ogg_mux->pulling;
-
+    GstOggPadData *pad = ogg_mux->pulling;
     GstClockTime last_ts = GST_BUFFER_END_TIME (pad->buffer);
 
     /* if the next packet in the current page is going to make the page
@@ -1201,7 +1206,7 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
       GST_LOG_OBJECT (pad->collect.pad,
           GST_GP_FORMAT " stored packet %" G_GINT64_FORMAT
           " will make page too long, flushing",
-          GST_BUFFER_OFFSET_END (pad->buffer), pad->stream.packetno);
+          GST_BUFFER_OFFSET_END (pad->buffer), (gint64) pad->stream.packetno);
 
       while (ogg_stream_flush (&pad->stream, &page)) {
         /* end time of this page is the timestamp of the next buffer */
@@ -1247,7 +1252,7 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
     ogg_packet packet;
     ogg_page page;
     GstBuffer *buf, *tmpbuf;
-    GstOggPad *pad = ogg_mux->pulling;
+    GstOggPadData *pad = ogg_mux->pulling;
     gint64 duration;
     gboolean force_flush;
 
@@ -1283,7 +1288,8 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
     packet.packetno = pad->packetno++;
     GST_LOG_OBJECT (pad->collect.pad, GST_GP_FORMAT
         " packet %" G_GINT64_FORMAT " (%ld bytes) created from buffer",
-        packet.granulepos, packet.packetno, packet.bytes);
+        GST_GP_CAST (packet.granulepos), (gint64) packet.packetno,
+        packet.bytes);
 
     packet.e_o_s = (pad->eos ? 1 : 0);
     tmpbuf = NULL;
@@ -1366,7 +1372,7 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
     GST_LOG_OBJECT (pad->collect.pad,
         GST_GP_FORMAT " packet %" G_GINT64_FORMAT ", gp time %"
         GST_TIME_FORMAT ", timestamp %" GST_TIME_FORMAT " packetin'd",
-        granulepos, packet.packetno, GST_TIME_ARGS (gp_time),
+        granulepos, (gint64) packet.packetno, GST_TIME_ARGS (gp_time),
         GST_TIME_ARGS (timestamp));
     /* don't need the old buffer anymore */
     gst_buffer_unref (pad->buffer);
@@ -1383,10 +1389,10 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
       GST_LOG_OBJECT (pad->collect.pad,
           GST_GP_FORMAT " packet %" G_GINT64_FORMAT ", time %"
           GST_TIME_FORMAT ") caused new page",
-          granulepos, packet.packetno, GST_TIME_ARGS (timestamp));
+          granulepos, (gint64) packet.packetno, GST_TIME_ARGS (timestamp));
       GST_LOG_OBJECT (pad->collect.pad,
-          GST_GP_FORMAT " new page %ld", ogg_page_granulepos (&page),
-          pad->stream.pageno);
+          GST_GP_FORMAT " new page %ld",
+          GST_GP_CAST (ogg_page_granulepos (&page)), pad->stream.pageno);
 
       if (ogg_page_granulepos (&page) == granulepos) {
         /* the packet we streamed in finishes on the current page,
@@ -1494,7 +1500,7 @@ beach:
 static GstFlowReturn
 gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
 {
-  GstOggPad *best;
+  GstOggPadData *best;
   GstFlowReturn ret;
   gint activebefore;
 
@@ -1590,7 +1596,7 @@ gst_ogg_mux_init_collectpads (GstCollectPads * collect)
 
   walk = collect->data;
   while (walk) {
-    GstOggPad *oggpad = (GstOggPad *) walk->data;
+    GstOggPadData *oggpad = (GstOggPadData *) walk->data;
 
     ogg_stream_init (&oggpad->stream, oggpad->serial);
     oggpad->packetno = 0;
@@ -1614,7 +1620,7 @@ gst_ogg_mux_clear_collectpads (GstCollectPads * collect)
   GSList *walk;
 
   for (walk = collect->data; walk; walk = g_slist_next (walk)) {
-    GstOggPad *oggpad = (GstOggPad *) walk->data;
+    GstOggPadData *oggpad = (GstOggPadData *) walk->data;
     GstBuffer *buf;
 
     ogg_stream_clear (&oggpad->stream);
