@@ -24,9 +24,11 @@
 
 #include "gstoggstream.h"
 #include "dirac_parse.h"
+#include "vorbis_parse.h"
 
 #include <gst/riff/riff-media.h>
 
+#include <stdlib.h>
 #include <string.h>
 
 GST_DEBUG_CATEGORY_EXTERN (gst_ogg_demux_debug);
@@ -53,11 +55,15 @@ typedef gboolean (*GstOggMapIsHeaderPacketFunc) (GstOggStream * pad,
     ogg_packet * packet);
 typedef gint64 (*GstOggMapPacketDurationFunc) (GstOggStream * pad,
     ogg_packet * packet);
+typedef void (*GstOggMapExtractTagsFunc) (GstOggStream * pad,
+    ogg_packet * packet);
 
-
+typedef gint64 (*GstOggMapGranuleposToKeyGranuleFunc) (GstOggStream * pad,
+    gint64 granulepos);
 
 #define SKELETON_FISBONE_MIN_SIZE  52
-
+#define SKELETON_FISHEAD_3_3_MIN_SIZE 112
+#define SKELETON_FISHEAD_4_0_MIN_SIZE 80
 
 struct _GstOggMap
 {
@@ -71,9 +77,11 @@ struct _GstOggMap
   GstOggMapIsKeyFrameFunc is_key_frame_func;
   GstOggMapIsHeaderPacketFunc is_header_func;
   GstOggMapPacketDurationFunc packet_duration_func;
+  GstOggMapGranuleposToKeyGranuleFunc granulepos_to_key_granule_func;
+  GstOggMapExtractTagsFunc extract_tags_func;
 };
 
-static const GstOggMap mappers[];
+extern const GstOggMap mappers[];
 
 GstClockTime
 gst_ogg_stream_get_packet_start_time (GstOggStream * pad, ogg_packet * packet)
@@ -141,6 +149,9 @@ gst_ogg_stream_granulepos_to_granule (GstOggStream * pad, gint64 granulepos)
 gint64
 gst_ogg_stream_granulepos_to_key_granule (GstOggStream * pad, gint64 granulepos)
 {
+  if (mappers[pad->map].granulepos_to_key_granule_func)
+    return mappers[pad->map].granulepos_to_key_granule_func (pad, granulepos);
+
   if (granulepos == -1 || granulepos == 0) {
     return granulepos;
   }
@@ -166,8 +177,7 @@ gst_ogg_stream_granule_to_granulepos (GstOggStream * pad, gint64 granule,
 }
 
 gboolean
-gst_ogg_stream_packet_granulepos_is_key_frame (GstOggStream * pad,
-    gint64 granulepos)
+gst_ogg_stream_granulepos_is_key_frame (GstOggStream * pad, gint64 granulepos)
 {
   if (granulepos == -1) {
     return FALSE;
@@ -204,7 +214,16 @@ gst_ogg_stream_get_packet_duration (GstOggStream * pad, ogg_packet * packet)
 }
 
 
+void
+gst_ogg_stream_extract_tags (GstOggStream * pad, ogg_packet * packet)
+{
+  if (mappers[pad->map].extract_tags_func == NULL) {
+    GST_DEBUG ("No tag extraction");
+    return;
+  }
 
+  mappers[pad->map].extract_tags_func (pad, packet);
+}
 
 /* some generic functions */
 
@@ -236,6 +255,11 @@ granule_to_granulepos_default (GstOggStream * pad, gint64 granule,
   gint64 keyoffset;
 
   if (pad->granuleshift != 0) {
+    /* If we don't know where the previous keyframe is yet, assume it is
+       at 0 or 1, depending on bitstream version. If nothing else, this
+       avoids getting negative granpos back. */
+    if (keyframe_granule < 0)
+      keyframe_granule = pad->theora_has_zero_keyoffset ? 0 : 1;
     keyoffset = granule - keyframe_granule;
     return (keyframe_granule << pad->granuleshift) | keyoffset;
   } else {
@@ -273,6 +297,49 @@ packet_duration_constant (GstOggStream * pad, ogg_packet * packet)
   return pad->frame_size;
 }
 
+/* helper: extracts tags from vorbis comment ogg packet.
+ * Returns result in *tags after free'ing existing *tags (if any) */
+static gboolean
+tag_list_from_vorbiscomment_packet (ogg_packet * packet,
+    const guint8 * id_data, const guint id_data_length, GstTagList ** tags)
+{
+  GstBuffer *buf = NULL;
+  gchar *encoder = NULL;
+  GstTagList *list;
+  gboolean ret = TRUE;
+
+  g_return_val_if_fail (tags != NULL, FALSE);
+
+  buf = gst_buffer_new ();
+  GST_BUFFER_DATA (buf) = (guint8 *) packet->packet;
+  GST_BUFFER_SIZE (buf) = packet->bytes;
+
+  list = gst_tag_list_from_vorbiscomment_buffer (buf, id_data, id_data_length,
+      &encoder);
+
+  if (!list) {
+    GST_WARNING ("failed to decode vorbis comments");
+    ret = FALSE;
+    goto exit;
+  }
+
+  if (encoder) {
+    if (encoder[0])
+      gst_tag_list_add (list, GST_TAG_MERGE_REPLACE, GST_TAG_ENCODER, encoder,
+          NULL);
+    g_free (encoder);
+  }
+
+exit:
+  if (*tags)
+    gst_tag_list_free (*tags);
+  *tags = list;
+
+  gst_buffer_unref (buf);
+
+  return ret;
+}
+
 /* theora */
 
 static gboolean
@@ -280,9 +347,14 @@ setup_theora_mapper (GstOggStream * pad, ogg_packet * packet)
 {
   guint8 *data = packet->packet;
   guint w, h, par_d, par_n;
+  guint8 vmaj, vmin, vrev;
 
-  w = GST_READ_UINT24_BE (data + 14) & 0xFFFFF0;
-  h = GST_READ_UINT24_BE (data + 17) & 0xFFFFF0;
+  vmaj = data[7];
+  vmin = data[8];
+  vrev = data[9];
+
+  w = GST_READ_UINT24_BE (data + 14) & 0xFFFFFF;
+  h = GST_READ_UINT24_BE (data + 17) & 0xFFFFFF;
 
   pad->granulerate_n = GST_READ_UINT32_BE (data + 22);
   pad->granulerate_d = GST_READ_UINT32_BE (data + 26);
@@ -297,14 +369,24 @@ setup_theora_mapper (GstOggStream * pad, ogg_packet * packet)
   pad->granuleshift = ((GST_READ_UINT8 (data + 40) & 0x03) << 3) +
       (GST_READ_UINT8 (data + 41) >> 5);
 
+  pad->is_video = TRUE;
   pad->n_header_packets = 3;
   pad->frame_size = 1;
 
+  pad->bitrate = GST_READ_UINT24_BE (data + 37);
+  GST_LOG ("bit rate: %d", pad->bitrate);
+
   if (pad->granulerate_n == 0 || pad->granulerate_d == 0) {
     GST_WARNING ("frame rate %d/%d", pad->granulerate_n, pad->granulerate_d);
     return FALSE;
   }
 
+  /* The interpretation of the granule position has changed with 3.2.1.
+     The granule is now made from the number of frames encoded, rather than
+     the index of the frame being encoded - so there is a difference of 1. */
+  pad->theora_has_zero_keyoffset =
+      ((vmaj << 16) | (vmin << 8) | vrev) < 0x030201;
+
   pad->caps = gst_caps_new_simple ("video/x-theora", NULL);
 
   if (w > 0 && h > 0) {
@@ -312,14 +394,14 @@ setup_theora_mapper (GstOggStream * pad, ogg_packet * packet)
         G_TYPE_INT, h, NULL);
   }
 
+  /* PAR of 0:N, N:0 and 0:0 is allowed and maps to 1:1 */
+  if (par_n == 0 || par_d == 0)
+    par_n = par_d = 1;
+
   /* only add framerate now so caps look prettier, with width/height first */
   gst_caps_set_simple (pad->caps, "framerate", GST_TYPE_FRACTION,
-      pad->granulerate_n, pad->granulerate_d, NULL);
-
-  if (par_n > 0 && par_d > 0) {
-    gst_caps_set_simple (pad->caps, "pixel-aspect-ratio", GST_TYPE_FRACTION,
-        par_n, par_d, NULL);
-  }
+      pad->granulerate_n, pad->granulerate_d, "pixel-aspect-ratio",
+      GST_TYPE_FRACTION, par_n, par_d, NULL);
 
   return TRUE;
 }
@@ -332,9 +414,6 @@ granulepos_to_granule_theora (GstOggStream * pad, gint64 granulepos)
   if (pad->granuleshift != 0) {
     keyindex = granulepos >> pad->granuleshift;
     keyoffset = granulepos - (keyindex << pad->granuleshift);
-    if (keyoffset == 0) {
-      pad->theora_has_zero_keyoffset = TRUE;
-    }
     if (pad->theora_has_zero_keyoffset) {
       keyoffset++;
     }
@@ -352,7 +431,7 @@ is_keyframe_theora (GstOggStream * pad, gint64 granulepos)
   if (granulepos == (gint64) - 1)
     return FALSE;
 
-  frame_mask = (1 << (pad->granuleshift + 1)) - 1;
+  frame_mask = (1 << pad->granuleshift) - 1;
 
   return ((granulepos & frame_mask) == 0);
 }
@@ -363,6 +442,22 @@ is_header_theora (GstOggStream * pad, ogg_packet * packet)
   return (packet->bytes > 0 && (packet->packet[0] & 0x80) == 0x80);
 }
 
+static void
+extract_tags_theora (GstOggStream * pad, ogg_packet * packet)
+{
+  if (packet->bytes > 0 && packet->packet[0] == 0x81) {
+    tag_list_from_vorbiscomment_packet (packet,
+        (const guint8 *) "\201theora", 7, &pad->taglist);
+
+    if (!pad->taglist)
+      pad->taglist = gst_tag_list_new ();
+
+    if (pad->bitrate)
+      gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+          GST_TAG_BITRATE, (guint) pad->bitrate, NULL);
+  }
+}
+
 /* dirac */
 
 static gboolean
@@ -378,6 +473,7 @@ setup_dirac_mapper (GstOggStream * pad, ogg_packet * packet)
     return FALSE;
   }
 
+  pad->is_video = TRUE;
   pad->granulerate_n = header.frame_rate_numerator * 2;
   pad->granulerate_d = header.frame_rate_denominator;
   pad->granuleshift = 22;
@@ -456,20 +552,164 @@ granule_to_granulepos_dirac (GstOggStream * pad, gint64 granule,
   return -1;
 }
 
+static gint64
+granulepos_to_key_granule_dirac (GstOggStream * pad, gint64 gp)
+{
+  gint64 pt;
+  int dist_h;
+  int dist_l;
+  int dist;
+  int delay;
+  gint64 dt;
+
+  if (gp == -1 || gp == 0)
+    return gp;
+
+  pt = ((gp >> 22) + (gp & OGG_DIRAC_GRANULE_LOW_MASK)) >> 9;
+  dist_h = (gp >> 22) & 0xff;
+  dist_l = gp & 0xff;
+  dist = (dist_h << 8) | dist_l;
+  delay = (gp >> 9) & 0x1fff;
+  dt = pt - delay;
+
+  return dt - 2 * dist + 4;
+}
+
+/* VP8 */
+
+static gboolean
+setup_vp8_mapper (GstOggStream * pad, ogg_packet * packet)
+{
+  gint width, height, par_n, par_d, fps_n, fps_d;
+
+  if (packet->bytes < 26) {
+    GST_DEBUG ("Failed to parse VP8 BOS page");
+    return FALSE;
+  }
+
+  width = GST_READ_UINT16_BE (packet->packet + 8);
+  height = GST_READ_UINT16_BE (packet->packet + 10);
+  par_n = GST_READ_UINT24_BE (packet->packet + 12);
+  par_d = GST_READ_UINT24_BE (packet->packet + 15);
+  fps_n = GST_READ_UINT32_BE (packet->packet + 18);
+  fps_d = GST_READ_UINT32_BE (packet->packet + 22);
+
+  pad->is_video = TRUE;
+  pad->is_vp8 = TRUE;
+  pad->granulerate_n = fps_n;
+  pad->granulerate_d = fps_d;
+  pad->n_header_packets = 2;
+  pad->frame_size = 1;
+
+  pad->caps = gst_caps_new_simple ("video/x-vp8",
+      "width", G_TYPE_INT, width,
+      "height", G_TYPE_INT, height,
+      "pixel-aspect-ratio", GST_TYPE_FRACTION,
+      par_n, par_d, "framerate", GST_TYPE_FRACTION, fps_n, fps_d, NULL);
+
+  return TRUE;
+}
+
+static gboolean
+is_keyframe_vp8 (GstOggStream * pad, gint64 granulepos)
+{
+  guint64 gpos = granulepos;
+
+  if (granulepos == -1)
+    return FALSE;
+
+  /* Get rid of flags */
+  gpos >>= 3;
+
+  return ((gpos & 0x07ffffff) == 0);
+}
+
+static gint64
+granulepos_to_granule_vp8 (GstOggStream * pad, gint64 gpos)
+{
+  guint64 gp = (guint64) gpos;
+  guint32 pt;
+  guint32 dist;
+
+  pt = (gp >> 32);
+  dist = (gp >> 3) & 0x07ffffff;
+
+  GST_DEBUG ("pt %u, dist %u", pt, dist);
+
+  return pt;
+}
+
+static gint64
+granule_to_granulepos_vp8 (GstOggStream * pad, gint64 granule,
+    gint64 keyframe_granule)
+{
+  /* FIXME: This requires to look into the content of the packets
+   * because the simple granule counter doesn't know about invisible
+   * frames...
+   */
+  return -1;
+}
+
+/* Check if this packet contains an invisible frame or not */
+static gint64
+packet_duration_vp8 (GstOggStream * pad, ogg_packet * packet)
+{
+  guint32 hdr;
+
+  if (packet->bytes < 3)
+    return 0;
+
+  hdr = GST_READ_UINT24_LE (packet->packet);
+
+  return (((hdr >> 4) & 1) != 0) ? 1 : 0;
+}
+
+static gint64
+granulepos_to_key_granule_vp8 (GstOggStream * pad, gint64 granulepos)
+{
+  guint64 gp = granulepos;
+  guint64 pts = (gp >> 32);
+  guint32 dist = (gp >> 3) & 0x07ffffff;
+
+  if (granulepos == -1 || granulepos == 0)
+    return granulepos;
+
+  if (dist > pts)
+    return 0;
+
+  return pts - dist;
+}
+
+static gboolean
+is_header_vp8 (GstOggStream * pad, ogg_packet * packet)
+{
+  if (packet->bytes >= 5 && packet->packet[0] == 0x4F &&
+      packet->packet[1] == 0x56 && packet->packet[2] == 0x50 &&
+      packet->packet[3] == 0x38 && packet->packet[4] == 0x30)
+    return TRUE;
+  return FALSE;
+}
+
+static void
+extract_tags_vp8 (GstOggStream * pad, ogg_packet * packet)
+{
+  if (packet->bytes >= 7 && memcmp (packet->packet, "OVP80\2 ", 7) == 0) {
+    tag_list_from_vorbiscomment_packet (packet,
+        (const guint8 *) "OVP80\2 ", 7, &pad->taglist);
+  }
+}
 
 /* vorbis */
 
-void parse_vorbis_header_packet (GstOggStream * pad, ogg_packet * op);
-void parse_vorbis_setup_packet (GstOggStream * pad, ogg_packet * op);
-
-
 static gboolean
 setup_vorbis_mapper (GstOggStream * pad, ogg_packet * packet)
 {
   guint8 *data = packet->packet;
   guint chans;
 
-  data += 1 + 6 + 4;
+  data += 1 + 6;
+  pad->version = GST_READ_UINT32_LE (data);
+  data += 4;
   chans = GST_READ_UINT8 (data);
   data += 1;
   pad->granulerate_n = GST_READ_UINT32_LE (data);
@@ -478,6 +718,24 @@ setup_vorbis_mapper (GstOggStream * pad, ogg_packet * packet)
   pad->last_size = 0;
   GST_LOG ("sample rate: %d", pad->granulerate_n);
 
+  data += 4;
+  pad->bitrate_upper = GST_READ_UINT32_LE (data);
+  data += 4;
+  pad->bitrate_nominal = GST_READ_UINT32_LE (data);
+  data += 4;
+  pad->bitrate_lower = GST_READ_UINT32_LE (data);
+
+  if (pad->bitrate_nominal > 0)
+    pad->bitrate = pad->bitrate_nominal;
+
+  if (pad->bitrate_upper > 0 && !pad->bitrate)
+    pad->bitrate = pad->bitrate_upper;
+
+  if (pad->bitrate_lower > 0 && !pad->bitrate)
+    pad->bitrate = pad->bitrate_lower;
+
+  GST_LOG ("bit rate: %d", pad->bitrate);
+
   pad->n_header_packets = 3;
 
   if (pad->granulerate_n == 0)
@@ -505,6 +763,40 @@ is_header_vorbis (GstOggStream * pad, ogg_packet * packet)
   return TRUE;
 }
 
+static void
+extract_tags_vorbis (GstOggStream * pad, ogg_packet * packet)
+{
+  if (packet->bytes == 0 || (packet->packet[0] & 0x01) == 0)
+    return;
+
+  if (((guint8 *) (packet->packet))[0] == 0x03) {
+    tag_list_from_vorbiscomment_packet (packet,
+        (const guint8 *) "\003vorbis", 7, &pad->taglist);
+
+    if (!pad->taglist)
+      pad->taglist = gst_tag_list_new ();
+
+    gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+        GST_TAG_ENCODER_VERSION, pad->version, NULL);
+
+    if (pad->bitrate_nominal > 0)
+      gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+          GST_TAG_NOMINAL_BITRATE, (guint) pad->bitrate_nominal, NULL);
+
+    if (pad->bitrate_upper > 0)
+      gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+          GST_TAG_MAXIMUM_BITRATE, (guint) pad->bitrate_upper, NULL);
+
+    if (pad->bitrate_lower > 0)
+      gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+          GST_TAG_MINIMUM_BITRATE, (guint) pad->bitrate_lower, NULL);
+
+    if (pad->bitrate)
+      gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+          GST_TAG_BITRATE, (guint) pad->bitrate, NULL);
+  }
+}
+
 static gint64
 packet_duration_vorbis (GstOggStream * pad, ogg_packet * packet)
 {
@@ -546,8 +838,11 @@ setup_speex_mapper (GstOggStream * pad, ogg_packet * packet)
 
   data += 4 + 4 + 4;
   chans = GST_READ_UINT32_LE (data);
+  data += 4;
+  pad->bitrate = GST_READ_UINT32_LE (data);
 
   GST_LOG ("sample rate: %d, channels: %u", pad->granulerate_n, chans);
+  GST_LOG ("bit rate: %d", pad->bitrate);
 
   pad->n_header_packets = GST_READ_UINT32_LE (packet->packet + 68) + 2;
   pad->frame_size = GST_READ_UINT32_LE (packet->packet + 64) *
@@ -562,6 +857,22 @@ setup_speex_mapper (GstOggStream * pad, ogg_packet * packet)
   return TRUE;
 }
 
+static void
+extract_tags_count (GstOggStream * pad, ogg_packet * packet)
+{
+  /* packet 2 must be comment packet */
+  if (packet->bytes > 0 && pad->n_header_packets_seen == 1) {
+    tag_list_from_vorbiscomment_packet (packet, NULL, 0, &pad->taglist);
+
+    if (!pad->taglist)
+      pad->taglist = gst_tag_list_new ();
+
+    if (pad->bitrate)
+      gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+          GST_TAG_BITRATE, (guint) pad->bitrate, NULL);
+  }
+}
+
 
 /* flac */
 
@@ -671,6 +982,15 @@ packet_duration_flac (GstOggStream * pad, ogg_packet * packet)
   return -1;
 }
 
+static void
+extract_tags_flac (GstOggStream * pad, ogg_packet * packet)
+{
+  if (packet->bytes > 4 && ((packet->packet[0] & 0x7F) == 0x4)) {
+    tag_list_from_vorbiscomment_packet (packet,
+        packet->packet, 4, &pad->taglist);
+  }
+}
+
 /* fishead */
 
 static gboolean
@@ -679,11 +999,15 @@ setup_fishead_mapper (GstOggStream * pad, ogg_packet * packet)
   guint8 *data;
   gint64 prestime_n, prestime_d;
   gint64 basetime_n, basetime_d;
-  gint64 basetime;
 
   data = packet->packet;
 
-  data += 8 + 2 + 2;            /* header + major/minor version */
+  data += 8;                    /* header */
+
+  pad->skeleton_major = GST_READ_UINT16_LE (data);
+  data += 2;
+  pad->skeleton_minor = GST_READ_UINT16_LE (data);
+  data += 2;
 
   prestime_n = (gint64) GST_READ_UINT64_LE (data);
   data += 8;
@@ -696,29 +1020,124 @@ setup_fishead_mapper (GstOggStream * pad, ogg_packet * packet)
 
   /* FIXME: we don't use basetime anywhere in the demuxer! */
   if (basetime_d != 0)
-    basetime = gst_util_uint64_scale (GST_SECOND, basetime_n, basetime_d);
+    pad->basetime = gst_util_uint64_scale (GST_SECOND, basetime_n, basetime_d);
   else
-    basetime = -1;
+    pad->basetime = -1;
 
-  GST_INFO ("skeleton fishead parsed (basetime: %" GST_TIME_FORMAT ")",
-      GST_TIME_ARGS (basetime));
+  if (prestime_d != 0)
+    pad->prestime = gst_util_uint64_scale (GST_SECOND, prestime_n, prestime_d);
+  else
+    pad->prestime = -1;
+
+  /* Ogg Skeleton 3.3+ streams provide additional information in the header */
+  if (packet->bytes >= SKELETON_FISHEAD_3_3_MIN_SIZE && pad->skeleton_major == 3
+      && pad->skeleton_minor > 0) {
+    gint64 firstsampletime_n, firstsampletime_d;
+    gint64 lastsampletime_n, lastsampletime_d;
+    gint64 firstsampletime, lastsampletime;
+    guint64 segment_length, content_offset;
+
+    firstsampletime_n = GST_READ_UINT64_LE (data + 64);
+    firstsampletime_d = GST_READ_UINT64_LE (data + 72);
+    lastsampletime_n = GST_READ_UINT64_LE (data + 80);
+    lastsampletime_d = GST_READ_UINT64_LE (data + 88);
+    segment_length = GST_READ_UINT64_LE (data + 96);
+    content_offset = GST_READ_UINT64_LE (data + 104);
+
+    GST_INFO ("firstsampletime %" G_GUINT64_FORMAT "/%" G_GUINT64_FORMAT,
+        firstsampletime_n, firstsampletime_d);
+    GST_INFO ("lastsampletime %" G_GUINT64_FORMAT "/%" G_GUINT64_FORMAT,
+        lastsampletime_n, lastsampletime_d);
+    GST_INFO ("segment length %" G_GUINT64_FORMAT, segment_length);
+    GST_INFO ("content offset %" G_GUINT64_FORMAT, content_offset);
+
+    if (firstsampletime_d > 0)
+      firstsampletime = gst_util_uint64_scale (GST_SECOND,
+          firstsampletime_n, firstsampletime_d);
+    else
+      firstsampletime = 0;
+
+    if (lastsampletime_d > 0)
+      lastsampletime = gst_util_uint64_scale (GST_SECOND,
+          lastsampletime_n, lastsampletime_d);
+    else
+      lastsampletime = 0;
+
+    if (lastsampletime > firstsampletime)
+      pad->total_time = lastsampletime - firstsampletime;
+    else
+      pad->total_time = -1;
+
+    GST_INFO ("skeleton fishead parsed total: %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (pad->total_time));
+  } else if (packet->bytes >= SKELETON_FISHEAD_4_0_MIN_SIZE
+      && pad->skeleton_major == 4) {
+    guint64 segment_length, content_offset;
+
+    segment_length = GST_READ_UINT64_LE (data + 64);
+    content_offset = GST_READ_UINT64_LE (data + 72);
+
+    GST_INFO ("segment length %" G_GUINT64_FORMAT, segment_length);
+    GST_INFO ("content offset %" G_GUINT64_FORMAT, content_offset);
+  } else {
+    pad->total_time = -1;
+  }
+
+  GST_INFO ("skeleton fishead %u.%u parsed (basetime: %" GST_TIME_FORMAT
+      ", prestime: %" GST_TIME_FORMAT ")", pad->skeleton_major,
+      pad->skeleton_minor, GST_TIME_ARGS (pad->basetime),
+      GST_TIME_ARGS (pad->prestime));
+
+  pad->is_skeleton = TRUE;
+  pad->is_sparse = TRUE;
+
+  pad->caps = gst_caps_new_simple ("none/none", NULL);
 
   return TRUE;
 }
 
 gboolean
-gst_ogg_map_add_fisbone (GstOggStream * pad,
-    const guint8 * data, guint size, GstClockTime * p_start_time,
-    guint32 * p_preroll)
+gst_ogg_map_parse_fisbone (GstOggStream * pad, const guint8 * data, guint size,
+    guint32 * serialno, GstOggSkeleton * type)
+{
+  GstOggSkeleton stype;
+  guint serial_offset;
+
+  if (size < SKELETON_FISBONE_MIN_SIZE) {
+    GST_WARNING ("small fisbone packet of size %d, ignoring", size);
+    return FALSE;
+  }
+
+  if (memcmp (data, "fisbone\0", 8) == 0) {
+    GST_INFO ("got fisbone packet");
+    stype = GST_OGG_SKELETON_FISBONE;
+    serial_offset = 12;
+  } else if (memcmp (data, "index\0", 6) == 0) {
+    GST_INFO ("got index packet");
+    stype = GST_OGG_SKELETON_INDEX;
+    serial_offset = 6;
+  } else if (memcmp (data, "fishead\0", 8) == 0) {
+    return FALSE;
+  } else {
+    GST_WARNING ("unknown skeleton packet \"%10.10s\"", data);
+    return FALSE;
+  }
+
+  if (serialno)
+    *serialno = GST_READ_UINT32_LE (data + serial_offset);
+
+  if (type)
+    *type = stype;
+
+  return TRUE;
+}
+
+gboolean
+gst_ogg_map_add_fisbone (GstOggStream * pad, GstOggStream * skel_pad,
+    const guint8 * data, guint size, GstClockTime * p_start_time)
 {
   GstClockTime start_time;
   gint64 start_granule;
-  guint32 preroll;
-
-  if (size < SKELETON_FISBONE_MIN_SIZE || memcmp (data, "fisbone\0", 8) != 0) {
-    GST_WARNING ("invalid fisbone packet, ignoring");
-    return FALSE;
-  }
 
   if (pad->have_fisbone) {
     GST_DEBUG ("already have fisbone, ignoring second one");
@@ -735,16 +1154,211 @@ gst_ogg_map_add_fisbone (GstOggStream * pad,
   pad->granulerate_d = GST_READ_UINT64_LE (data + 8);
 
   start_granule = GST_READ_UINT64_LE (data + 16);
-  preroll = GST_READ_UINT32_LE (data + 24);
+  pad->preroll = GST_READ_UINT32_LE (data + 24);
   pad->granuleshift = GST_READ_UINT8 (data + 28);
 
   start_time = granulepos_to_granule_default (pad, start_granule);
 
+  GST_INFO ("skeleton fisbone parsed "
+      "(start time: %" GST_TIME_FORMAT
+      " granulerate_n: %d granulerate_d: %d "
+      " preroll: %" G_GUINT32_FORMAT " granuleshift: %d)",
+      GST_TIME_ARGS (start_time),
+      pad->granulerate_n, pad->granulerate_d, pad->preroll, pad->granuleshift);
+
   if (p_start_time)
     *p_start_time = start_time;
 
-  if (p_preroll)
-    *p_preroll = preroll;
+  return TRUE;
+}
+
+static gboolean
+read_vlc (const guint8 ** data, guint * size, guint64 * result)
+{
+  gint shift = 0;
+  guint8 byte;
+
+  *result = 0;
+
+  do {
+    if (G_UNLIKELY (*size < 1))
+      return FALSE;
+
+    byte = **data;
+    *result |= ((byte & 0x7f) << shift);
+    shift += 7;
+
+    (*data)++;
+    (*size)--;
+  } while ((byte & 0x80) != 0x80);
+
+  return TRUE;
+}
+
+gboolean
+gst_ogg_map_add_index (GstOggStream * pad, GstOggStream * skel_pad,
+    const guint8 * data, guint size)
+{
+  guint64 i, n_keypoints, isize;
+  guint64 offset, timestamp;
+  guint64 offset_d, timestamp_d;
+
+  if (pad->index) {
+    GST_DEBUG ("already have index, ignoring second one");
+    return TRUE;
+  }
+
+  if ((skel_pad->skeleton_major == 3 && size < 26) ||
+      (skel_pad->skeleton_major == 4 && size < 62)) {
+    GST_WARNING ("small index packet of size %u, ignoring", size);
+    return FALSE;
+  }
+
+  /* skip "index\0" + serialno */
+  data += 6 + 4;
+  size -= 6 + 4;
+
+  n_keypoints = GST_READ_UINT64_LE (data);
+
+  data += 8;
+  size -= 8;
+
+  pad->kp_denom = GST_READ_UINT64_LE (data);
+  if (pad->kp_denom == 0)
+    pad->kp_denom = 1;
+
+  data += 8;
+  size -= 8;
+
+  if (skel_pad->skeleton_major == 4) {
+    gint64 firstsampletime_n;
+    gint64 lastsampletime_n;
+    gint64 firstsampletime, lastsampletime;
+
+    firstsampletime_n = GST_READ_UINT64_LE (data + 0);
+    lastsampletime_n = GST_READ_UINT64_LE (data + 8);
+
+    GST_INFO ("firstsampletime %" G_GUINT64_FORMAT "/%" G_GUINT64_FORMAT,
+        firstsampletime_n, pad->kp_denom);
+    GST_INFO ("lastsampletime %" G_GUINT64_FORMAT "/%" G_GUINT64_FORMAT,
+        lastsampletime_n, pad->kp_denom);
+
+    firstsampletime = gst_util_uint64_scale (GST_SECOND,
+        firstsampletime_n, pad->kp_denom);
+    lastsampletime = gst_util_uint64_scale (GST_SECOND,
+        lastsampletime_n, pad->kp_denom);
+
+    if (lastsampletime > firstsampletime)
+      pad->total_time = lastsampletime - firstsampletime;
+    else
+      pad->total_time = -1;
+
+    GST_INFO ("skeleton index parsed total: %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (pad->total_time));
+
+    data += 16;
+    size -= 16;
+  }
+
+  GST_INFO ("skeleton index has %" G_GUINT64_FORMAT " keypoints, denom: %"
+      G_GINT64_FORMAT, n_keypoints, pad->kp_denom);
+
+  pad->index = g_try_new (GstOggIndex, n_keypoints);
+  if (!pad->index)
+    return FALSE;
+
+  isize = 0;
+  offset = 0;
+  timestamp = 0;
+
+  for (i = 0; i < n_keypoints; i++) {
+    /* read deltas */
+    if (!read_vlc (&data, &size, &offset_d))
+      break;
+    if (!read_vlc (&data, &size, &timestamp_d))
+      break;
+
+    offset += offset_d;
+    timestamp += timestamp_d;
+
+    pad->index[i].offset = offset;
+    pad->index[i].timestamp = timestamp;
+    isize++;
+
+    GST_INFO ("offset %" G_GUINT64_FORMAT " time %" G_GUINT64_FORMAT, offset,
+        timestamp);
+  }
+  if (isize != n_keypoints) {
+    GST_WARNING ("truncated index, expected %" G_GUINT64_FORMAT ", found %"
+        G_GUINT64_FORMAT, n_keypoints, isize);
+  }
+  pad->n_index = isize;
+  /* try to use the index to estimate the bitrate */
+  if (isize > 2) {
+    guint64 so, eo, st, et, b, t;
+
+    /* get start and end offset and timestamps */
+    so = pad->index[0].offset;
+    st = pad->index[0].timestamp;
+    eo = pad->index[isize - 1].offset;
+    et = pad->index[isize - 1].timestamp;
+
+    b = eo - so;
+    t = et - st;
+
+    GST_DEBUG ("bytes/time %" G_GUINT64_FORMAT "/%" G_GUINT64_FORMAT, b, t);
+
+    /* this is the total stream bitrate according to this index */
+    pad->idx_bitrate = gst_util_uint64_scale (8 * b, pad->kp_denom, t);
+
+    GST_DEBUG ("bitrate %" G_GUINT64_FORMAT, pad->idx_bitrate);
+  }
+
+  return TRUE;
+}
+
+static gint
+gst_ogg_index_compare (const GstOggIndex * index, const guint64 * ts,
+    gpointer user_data)
+{
+  if (index->timestamp < *ts)
+    return -1;
+  else if (index->timestamp > *ts)
+    return 1;
+  else
+    return 0;
+}
+
+gboolean
+gst_ogg_map_search_index (GstOggStream * pad, gboolean before,
+    guint64 * timestamp, guint64 * offset)
+{
+  guint64 n_index;
+  guint64 ts;
+  GstOggIndex *best;
+
+  n_index = pad->n_index;
+  if (n_index == 0 || pad->index == NULL)
+    return FALSE;
+
+  ts = gst_util_uint64_scale (*timestamp, pad->kp_denom, GST_SECOND);
+  GST_INFO ("timestamp %" G_GUINT64_FORMAT, ts);
+
+  best =
+      gst_util_array_binary_search (pad->index, n_index, sizeof (GstOggIndex),
+      (GCompareDataFunc) gst_ogg_index_compare, GST_SEARCH_MODE_BEFORE, &ts,
+      NULL);
+
+  if (best == NULL)
+    return FALSE;
+
+  GST_INFO ("found at index %u", (guint) (best - pad->index));
+
+  if (offset)
+    *offset = best->offset;
+  if (timestamp)
+    *timestamp =
+        gst_util_uint64_scale (best->timestamp, GST_SECOND, pad->kp_denom);
 
   return TRUE;
 }
@@ -767,6 +1381,15 @@ is_header_ogm (GstOggStream * pad, ogg_packet * packet)
   return FALSE;
 }
 
+static void
+extract_tags_ogm (GstOggStream * pad, ogg_packet * packet)
+{
+  if (!(packet->packet[0] & 1) && (packet->packet[0] & 3 && pad->is_ogm_text)) {
+    tag_list_from_vorbiscomment_packet (packet,
+        (const guint8 *) "\003vorbis", 7, &pad->taglist);
+  }
+}
+
 static gint64
 packet_duration_ogm (GstOggStream * pad, ogg_packet * packet)
 {
@@ -835,6 +1458,7 @@ setup_ogmvideo_mapper (GstOggStream * pad, ogg_packet * packet)
   GST_DEBUG ("time unit %d", GST_READ_UINT32_LE (data + 16));
   GST_DEBUG ("samples per unit %d", GST_READ_UINT32_LE (data + 24));
 
+  pad->is_video = TRUE;
   pad->granulerate_n = 10000000;
   time_unit = GST_READ_UINT64_LE (data + 17);
   if (time_unit > G_MAXINT || time_unit < G_MININT) {
@@ -898,6 +1522,7 @@ setup_ogmtext_mapper (GstOggStream * pad, ogg_packet * packet)
   pad->n_header_packets = 1;
   pad->is_ogm = TRUE;
   pad->is_ogm_text = TRUE;
+  pad->is_sparse = TRUE;
 
   return TRUE;
 }
@@ -1055,6 +1680,7 @@ setup_cmml_mapper (GstOggStream * pad, ogg_packet * packet)
   GST_DEBUG ("blocksize1: %u", 1 << (data[0] & 0x0F));
 
   pad->caps = gst_caps_new_simple ("text/x-cmml", NULL);
+  pad->is_sparse = TRUE;
 
   return TRUE;
 }
@@ -1100,6 +1726,7 @@ setup_kate_mapper (GstOggStream * pad, ogg_packet * packet)
   GST_LOG ("sample rate: %d", pad->granulerate_n);
 
   pad->n_header_packets = GST_READ_UINT8 (data + 11);
+  GST_LOG ("kate header packets: %d", pad->n_header_packets);
 
   if (pad->granulerate_n == 0)
     return FALSE;
@@ -1113,13 +1740,92 @@ setup_kate_mapper (GstOggStream * pad, ogg_packet * packet)
     pad->caps = gst_caps_new_simple ("application/x-kate", NULL);
   }
 
+  pad->is_sparse = TRUE;
+
   return TRUE;
 }
 
+static gint64
+packet_duration_kate (GstOggStream * pad, ogg_packet * packet)
+{
+  gint64 duration;
+
+  if (packet->bytes < 1)
+    return 0;
+
+  switch (packet->packet[0]) {
+    case 0x00:                 /* text data */
+      if (packet->bytes < 1 + 8 * 2) {
+        duration = 0;
+      } else {
+        duration = GST_READ_UINT64_LE (packet->packet + 1 + 8);
+        if (duration < 0)
+          duration = 0;
+      }
+      break;
+    default:
+      duration = GST_CLOCK_TIME_NONE;
+      break;
+  }
+
+  return duration;
+}
+
+static void
+extract_tags_kate (GstOggStream * pad, ogg_packet * packet)
+{
+  GstTagList *list = NULL;
+
+  if (packet->bytes <= 0)
+    return;
+
+  switch (packet->packet[0]) {
+    case 0x80:{
+      const gchar *canonical;
+      char language[16];
+
+      if (packet->bytes < 64) {
+        GST_WARNING ("Kate ID header packet is less than 64 bytes, ignored");
+        break;
+      }
+
+      /* the language tag is 16 bytes at offset 32, ensure NUL terminator */
+      memcpy (language, packet->packet + 32, 16);
+      language[15] = 0;
+
+      /* language is an ISO 639-1 code or RFC 3066 language code, we
+       * truncate to ISO 639-1 */
+      g_strdelimit (language, NULL, '\0');
+      canonical = gst_tag_get_language_code_iso_639_1 (language);
+      if (canonical) {
+        list = gst_tag_list_new_full (GST_TAG_LANGUAGE_CODE, canonical, NULL);
+      } else {
+        GST_WARNING ("Unknown or invalid language code %s, ignored", language);
+      }
+      break;
+    }
+    case 0x81:
+      tag_list_from_vorbiscomment_packet (packet,
+          (const guint8 *) "\201kate\0\0\0\0", 9, &list);
+      break;
+    default:
+      break;
+  }
+
+  if (list) {
+    if (pad->taglist) {
+      /* ensure the comment packet cannot override the category/language
+         from the identification header */
+      gst_tag_list_insert (pad->taglist, list, GST_TAG_MERGE_KEEP_ALL);
+    } else
+      pad->taglist = list;
+  }
+}
+
 
 /* *INDENT-OFF* */
 /* indent hates our freedoms */
-static const GstOggMap mappers[] = {
+const GstOggMap mappers[] = {
   {
     "\200theora", 7, 42,
     "video/x-theora",
@@ -1128,7 +1834,9 @@ static const GstOggMap mappers[] = {
     granule_to_granulepos_default,
     is_keyframe_theora,
     is_header_theora,
-    packet_duration_constant
+    packet_duration_constant,
+    NULL,
+    extract_tags_theora
   },
   {
     "\001vorbis", 7, 22,
@@ -1138,7 +1846,9 @@ static const GstOggMap mappers[] = {
     granule_to_granulepos_default,
     is_keyframe_true,
     is_header_vorbis,
-    packet_duration_vorbis
+    packet_duration_vorbis,
+    NULL,
+    extract_tags_vorbis
   },
   {
     "Speex", 5, 80,
@@ -1148,7 +1858,9 @@ static const GstOggMap mappers[] = {
     granule_to_granulepos_default,
     is_keyframe_true,
     is_header_count,
-    packet_duration_constant
+    packet_duration_constant,
+    NULL,
+    extract_tags_count
   },
   {
     "PCM     ", 8, 0,
@@ -1158,6 +1870,8 @@ static const GstOggMap mappers[] = {
     NULL,
     NULL,
     is_header_count,
+    NULL,
+    NULL,
     NULL
   },
   {
@@ -1168,6 +1882,8 @@ static const GstOggMap mappers[] = {
     NULL,
     NULL,
     is_header_count,
+    NULL,
+    NULL,
     NULL
   },
   {
@@ -1178,6 +1894,8 @@ static const GstOggMap mappers[] = {
     granule_to_granulepos_default,
     NULL,
     is_header_count,
+    NULL,
+    NULL,
     NULL
   },
   {
@@ -1188,6 +1906,8 @@ static const GstOggMap mappers[] = {
     NULL,
     NULL,
     is_header_true,
+    NULL,
+    NULL,
     NULL
   },
   {
@@ -1198,6 +1918,8 @@ static const GstOggMap mappers[] = {
     granule_to_granulepos_default,
     is_keyframe_true,
     is_header_fLaC,
+    packet_duration_flac,
+    NULL,
     NULL
   },
   {
@@ -1208,7 +1930,9 @@ static const GstOggMap mappers[] = {
     granule_to_granulepos_default,
     is_keyframe_true,
     is_header_flac,
-    packet_duration_flac
+    packet_duration_flac,
+    NULL,
+    extract_tags_flac
   },
   {
     "AnxData", 7, 0,
@@ -1218,6 +1942,8 @@ static const GstOggMap mappers[] = {
     NULL,
     NULL,
     NULL,
+    NULL,
+    NULL
   },
   {
     "CELT    ", 8, 0,
@@ -1227,7 +1953,9 @@ static const GstOggMap mappers[] = {
     granule_to_granulepos_default,
     NULL,
     is_header_count,
-    packet_duration_constant
+    packet_duration_constant,
+    NULL,
+    extract_tags_count
   },
   {
     "\200kate\0\0\0", 8, 0,
@@ -1237,7 +1965,9 @@ static const GstOggMap mappers[] = {
     granule_to_granulepos_default,
     NULL,
     is_header_count,
-    NULL
+    packet_duration_kate,
+    NULL,
+    extract_tags_kate
   },
   {
     "BBCD\0", 5, 13,
@@ -1247,7 +1977,21 @@ static const GstOggMap mappers[] = {
     granule_to_granulepos_dirac,
     is_keyframe_dirac,
     is_header_count,
-    packet_duration_constant
+    packet_duration_constant,
+    granulepos_to_key_granule_dirac,
+    NULL
+  },
+  {
+    "OVP80\1\1", 7, 4,
+    "video/x-vp8",
+    setup_vp8_mapper,
+    granulepos_to_granule_vp8,
+    granule_to_granulepos_vp8,
+    is_keyframe_vp8,
+    is_header_vp8,
+    packet_duration_vp8,
+    granulepos_to_key_granule_vp8,
+    extract_tags_vp8
   },
   {
     "\001audio\0\0\0", 9, 53,
@@ -1257,7 +2001,9 @@ static const GstOggMap mappers[] = {
     granule_to_granulepos_default,
     is_keyframe_true,
     is_header_ogm,
-    packet_duration_ogm
+    packet_duration_ogm,
+    NULL,
+    NULL
   },
   {
     "\001video\0\0\0", 9, 53,
@@ -1267,7 +2013,9 @@ static const GstOggMap mappers[] = {
     granule_to_granulepos_default,
     NULL,
     is_header_ogm,
-    packet_duration_constant
+    packet_duration_constant,
+    NULL,
+    NULL
   },
   {
     "\001text\0\0\0", 9, 9,
@@ -1277,7 +2025,9 @@ static const GstOggMap mappers[] = {
     granule_to_granulepos_default,
     is_keyframe_true,
     is_header_ogm,
-    packet_duration_ogm
+    packet_duration_ogm,
+    NULL,
+    extract_tags_ogm
   }
 };
 /* *INDENT-ON* */
