@@ -178,8 +178,8 @@ gst_rtp_buffer_new_copy_data (gpointer data, guint len)
  * @pad_len: the amount of padding
  * @csrc_count: the number of CSRC entries
  *
- * Allocate a new #Gstbuffer with enough data to hold an RTP packet with @csrc_count CSRCs,
- * a payload length of @payload_len and padding of @pad_len.
+ * Allocate a new #GstBuffer with enough data to hold an RTP packet with
+ * @csrc_count CSRCs, a payload length of @payload_len and padding of @pad_len.
  * All other RTP header fields will be set to 0/FALSE.
  *
  * Returns: A newly allocated buffer that can hold an RTP packet with given
@@ -359,16 +359,21 @@ validate_data (guint8 * data, guint len, guint8 * payload, guint payload_len)
 wrong_length:
   {
     GST_DEBUG ("len < header_len check failed (%d < %d)", len, header_len);
-    return FALSE;
+    goto dump_packet;
   }
 wrong_version:
   {
     GST_DEBUG ("version check failed (%d != %d)", version, GST_RTP_VERSION);
-    return FALSE;
+    goto dump_packet;
   }
 wrong_padding:
   {
     GST_DEBUG ("padding check failed (%d - %d < %d)", len, header_len, padding);
+    goto dump_packet;
+  }
+dump_packet:
+  {
+    GST_MEMDUMP ("buffer", data, len);
     return FALSE;
   }
 }
@@ -448,33 +453,20 @@ gst_rtp_buffer_list_validate (GstBufferList * list)
     guint8 *packet_payload;
     guint payload_size;
     guint packet_size;
+    guint j, n_buffers;
 
-    /* each group should consists of 2 buffers: one containing the RTP header
-     * and the other one the payload, FIXME, relax the requirement of only one
-     * payload buffer. */
-    if (gst_buffer_list_iterator_n_buffers (it) != 2)
+    /* each group should consists of at least 1 buffer: The first buffer always
+     * contains the complete RTP header. Next buffers contain the payload */
+    n_buffers = gst_buffer_list_iterator_n_buffers (it);
+    if (n_buffers < 1)
       goto invalid_list;
 
-    /* get the RTP header */
+    /* get the RTP header (and if n_buffers == 1 also the payload) */
     rtpbuf = gst_buffer_list_iterator_next (it);
     packet_header = GST_BUFFER_DATA (rtpbuf);
     if (packet_header == NULL)
       goto invalid_list;
 
-    /* get the payload */
-    paybuf = gst_buffer_list_iterator_next (it);
-    packet_payload = GST_BUFFER_DATA (paybuf);
-    if (packet_payload == NULL) {
-      goto invalid_list;
-    }
-    payload_size = GST_BUFFER_SIZE (paybuf);
-    if (payload_size == 0) {
-      goto invalid_list;
-    }
-
-    /* the size of the RTP packet within the current group */
-    packet_size = GST_BUFFER_SIZE (rtpbuf) + payload_size;
-
     /* check the sequence number */
     if (G_UNLIKELY (i == 0)) {
       prev_seqnum = g_ntohs (GST_RTP_HEADER_SEQ (packet_header));
@@ -484,6 +476,25 @@ gst_rtp_buffer_list_validate (GstBufferList * list)
         goto invalid_list;
     }
 
+    packet_size = GST_BUFFER_SIZE (rtpbuf);
+    packet_payload = NULL;
+    payload_size = 0;
+
+    /* get the payload buffers */
+    for (j = 1; j < n_buffers; j++) {
+      /* get the payload */
+      paybuf = gst_buffer_list_iterator_next (it);
+
+      if ((packet_payload = GST_BUFFER_DATA (paybuf)) == NULL)
+        goto invalid_list;
+
+      if ((payload_size = GST_BUFFER_SIZE (paybuf)) == 0)
+        goto invalid_list;
+
+      /* the size of the RTP packet within the current group */
+      packet_size += payload_size;
+    }
+
     /* validate packet */
     if (!validate_data (packet_header, packet_size, packet_payload,
             payload_size)) {
@@ -734,7 +745,7 @@ gst_rtp_buffer_get_extension_data (GstBuffer * buffer, guint16 * bits,
  *
  * Returns: True if done.
  *
- * Since : 0.10.18
+ * Since: 0.10.18
  */
 gboolean
 gst_rtp_buffer_set_extension_data (GstBuffer * buffer, guint16 bits,
@@ -1381,3 +1392,674 @@ gst_rtp_buffer_ext_timestamp (guint64 * exttimestamp, guint32 timestamp)
 
   return result;
 }
+
+/**
+ * gst_rtp_buffer_get_extension_onebyte_header:
+ * @buffer: the buffer
+ * @id: The ID of the header extension to be read (between 1 and 14).
+ * @nth: Read the nth extension packet with the requested ID
+ * @data: location for data
+ * @size: the size of the data in bytes
+ *
+ * Parses RFC 5285 style header extensions with a one byte header. It will
+ * return the nth extension with the requested id.
+ *
+ * Returns: TRUE if @buffer had the requested header extension
+ *
+ * Since: 0.10.31
+ */
+
+gboolean
+gst_rtp_buffer_get_extension_onebyte_header (GstBuffer * buffer, guint8 id,
+    guint nth, gpointer * data, guint * size)
+{
+  guint16 bits;
+  guint8 *pdata;
+  guint wordlen;
+  gulong offset = 0;
+  guint count = 0;
+
+  g_return_val_if_fail (id > 0 && id < 15, FALSE);
+
+  if (!gst_rtp_buffer_get_extension_data (buffer, &bits, (gpointer) & pdata,
+          &wordlen))
+    return FALSE;
+
+  if (bits != 0xBEDE)
+    return FALSE;
+
+  for (;;) {
+    guint8 read_id, read_len;
+
+    if (offset + 1 >= wordlen * 4)
+      break;
+
+    read_id = GST_READ_UINT8 (pdata + offset) >> 4;
+    read_len = (GST_READ_UINT8 (pdata + offset) & 0x0F) + 1;
+    offset += 1;
+
+    /* ID 0 means its padding, skip */
+    if (read_id == 0)
+      continue;
+
+    /* ID 15 is special and means we should stop parsing */
+    if (read_id == 15)
+      break;
+
+    /* Ignore extension headers where the size does not fit */
+    if (offset + read_len > wordlen * 4)
+      break;
+
+    /* If we have the right one */
+    if (id == read_id) {
+      if (nth == count) {
+        if (data)
+          *data = pdata + offset;
+        if (size)
+          *size = read_len;
+
+        return TRUE;
+      }
+
+      count++;
+    }
+    offset += read_len;
+
+    if (offset >= wordlen * 4)
+      break;
+  }
+
+  return FALSE;
+}
+
+/**
+ * gst_rtp_buffer_get_extension_twobytes_header:
+ * @buffer: the buffer
+ * @appbits: Application specific bits
+ * @id: The ID of the header extension to be read (between 1 and 14).
+ * @nth: Read the nth extension packet with the requested ID
+ * @data: location for data
+ * @size: the size of the data in bytes
+ *
+ * Parses RFC 5285 style header extensions with a two bytes header. It will
+ * return the nth extension with the requested id.
+ *
+ * Returns: TRUE if @buffer had the requested header extension
+ *
+ * Since: 0.10.31
+ */
+
+gboolean
+gst_rtp_buffer_get_extension_twobytes_header (GstBuffer * buffer,
+    guint8 * appbits, guint8 id, guint nth, gpointer * data, guint * size)
+{
+  guint16 bits;
+  guint8 *pdata;
+  guint wordlen;
+  guint bytelen;
+  gulong offset = 0;
+  guint count = 0;
+
+  if (!gst_rtp_buffer_get_extension_data (buffer, &bits, (gpointer) & pdata,
+          &wordlen))
+    return FALSE;
+
+  if (bits >> 4 != 0x100)
+    return FALSE;
+
+  bytelen = wordlen * 4;
+
+  for (;;) {
+    guint8 read_id, read_len;
+
+    if (offset + 2 >= bytelen)
+      break;
+
+    read_id = GST_READ_UINT8 (pdata + offset);
+    offset += 1;
+
+    if (read_id == 0)
+      continue;
+
+    read_len = GST_READ_UINT8 (pdata + offset);
+    offset += 1;
+
+    /* Ignore extension headers where the size does not fit */
+    if (offset + read_len > bytelen)
+      break;
+
+    /* If we have the right one, return it */
+    if (id == read_id) {
+      if (nth == count) {
+        if (data)
+          *data = pdata + offset;
+        if (size)
+          *size = read_len;
+        if (appbits)
+          *appbits = bits;
+
+        return TRUE;
+      }
+
+      count++;
+    }
+    offset += read_len;
+  }
+
+  return FALSE;
+}
+
+static guint
+get_onebyte_header_end_offset (guint8 * pdata, guint wordlen)
+{
+  guint offset = 0;
+  guint bytelen = wordlen * 4;
+  guint paddingcount = 0;
+
+  while (offset + 1 < bytelen) {
+    guint8 read_id, read_len;
+
+    read_id = GST_READ_UINT8 (pdata + offset) >> 4;
+    read_len = (GST_READ_UINT8 (pdata + offset) & 0x0F) + 1;
+    offset += 1;
+
+    /* ID 0 means its padding, skip */
+    if (read_id == 0) {
+      paddingcount++;
+      continue;
+    }
+
+    paddingcount = 0;
+
+    /* ID 15 is special and means we should stop parsing */
+    /* It also means we can't add an extra packet */
+    if (read_id == 15)
+      return 0;
+
+    /* Ignore extension headers where the size does not fit */
+    if (offset + read_len > bytelen)
+      return 0;
+
+    offset += read_len;
+  }
+
+  return offset - paddingcount;
+}
+
+/**
+ * gst_rtp_buffer_add_extension_onebyte_header:
+ * @buffer: the buffer
+ * @id: The ID of the header extension (between 1 and 14).
+ * @data: location for data
+ * @size: the size of the data in bytes
+ *
+ * Adds a RFC 5285 header extension with a one byte header to the end of the
+ * RTP header. If there is already a RFC 5285 header extension with a one byte
+ * header, the new extension will be appended.
+ * It will not work if there is already a header extension that does not follow
+ * the mecanism described in RFC 5285 or if there is a header extension with
+ * a two bytes header as described in RFC 5285. In that case, use
+ * gst_rtp_buffer_add_extension_twobytes_header()
+ *
+ * Returns: %TRUE if header extension could be added
+ *
+ * Since: 0.10.31
+ */
+
+gboolean
+gst_rtp_buffer_add_extension_onebyte_header (GstBuffer * buffer, guint8 id,
+    gpointer data, guint size)
+{
+  guint16 bits;
+  guint8 *pdata;
+  guint wordlen;
+  gboolean has_bit;
+  guint bytelen;
+
+  g_return_val_if_fail (id > 0 && id < 15, FALSE);
+  g_return_val_if_fail (size >= 1 && size <= 16, FALSE);
+  g_return_val_if_fail (gst_buffer_is_writable (buffer), FALSE);
+
+  has_bit = gst_rtp_buffer_get_extension_data (buffer, &bits,
+      (gpointer) & pdata, &wordlen);
+
+  bytelen = wordlen * 4;
+
+  if (has_bit) {
+    gulong offset = 0;
+    guint8 *nextext;
+    guint extlen;
+
+    if (bits != 0xBEDE)
+      return FALSE;
+
+    offset = get_onebyte_header_end_offset (pdata, wordlen);
+    if (offset == 0)
+      return FALSE;
+
+    nextext = pdata + offset;
+    offset = nextext - GST_BUFFER_DATA (buffer);
+
+    /* Don't add extra header if there isn't enough space */
+    if (GST_BUFFER_SIZE (buffer) < offset + size + 1)
+      return FALSE;
+
+    nextext[0] = (id << 4) | (0x0F & (size - 1));
+    memcpy (nextext + 1, data, size);
+
+    extlen = nextext - pdata + size + 1;
+    if (extlen % 4) {
+      wordlen = extlen / 4 + 1;
+      memset (nextext + size + 1, 0, 4 - extlen % 4);
+    } else {
+      wordlen = extlen / 4;
+    }
+
+    gst_rtp_buffer_set_extension_data (buffer, 0xBEDE, wordlen);
+  } else {
+    wordlen = (size + 1) / 4 + (((size + 1) % 4) ? 1 : 0);
+
+    gst_rtp_buffer_set_extension_data (buffer, 0xBEDE, wordlen);
+
+    gst_rtp_buffer_get_extension_data (buffer, &bits,
+        (gpointer) & pdata, &wordlen);
+
+    pdata[0] = (id << 4) | (0x0F & (size - 1));
+    memcpy (pdata + 1, data, size);
+
+    if ((size + 1) % 4)
+      memset (pdata + size + 1, 0, 4 - ((size + 1) % 4));
+  }
+
+  return TRUE;
+}
+
+
+static guint
+get_twobytes_header_end_offset (guint8 * pdata, guint wordlen)
+{
+  guint offset = 0;
+  guint bytelen = wordlen * 4;
+  guint paddingcount = 0;
+
+  while (offset + 2 < bytelen) {
+    guint8 read_id, read_len;
+
+    read_id = GST_READ_UINT8 (pdata + offset);
+    offset += 1;
+
+    /* ID 0 means its padding, skip */
+    if (read_id == 0) {
+      paddingcount++;
+      continue;
+    }
+
+    paddingcount = 0;
+
+    read_len = GST_READ_UINT8 (pdata + offset);
+    offset += 1;
+
+    /* Ignore extension headers where the size does not fit */
+    if (offset + read_len > bytelen)
+      return 0;
+
+    offset += read_len;
+  }
+
+  return offset - paddingcount;
+}
+
+/**
+ * gst_rtp_buffer_add_extension_twobytes_header:
+ * @buffer: the buffer
+ * @appbits: Application specific bits
+ * @id: The ID of the header extension
+ * @data: location for data
+ * @size: the size of the data in bytes
+ *
+ * Adds a RFC 5285 header extension with a two bytes header to the end of the
+ * RTP header. If there is already a RFC 5285 header extension with a two bytes
+ * header, the new extension will be appended.
+ * It will not work if there is already a header extension that does not follow
+ * the mecanism described in RFC 5285 or if there is a header extension with
+ * a one byte header as described in RFC 5285. In that case, use
+ * gst_rtp_buffer_add_extension_onebyte_header()
+ *
+ * Returns: %TRUE if header extension could be added
+ *
+ * Since: 0.10.31
+ */
+
+gboolean
+gst_rtp_buffer_add_extension_twobytes_header (GstBuffer * buffer,
+    guint8 appbits, guint8 id, gpointer data, guint size)
+{
+  guint16 bits;
+  guint8 *pdata;
+  guint wordlen;
+  gboolean has_bit;
+  guint bytelen;
+
+  g_return_val_if_fail ((appbits & 0xF0) == 0, FALSE);
+  g_return_val_if_fail (size < 256, FALSE);
+  g_return_val_if_fail (gst_buffer_is_writable (buffer), FALSE);
+
+  has_bit = gst_rtp_buffer_get_extension_data (buffer, &bits,
+      (gpointer) & pdata, &wordlen);
+
+  bytelen = wordlen * 4;
+
+  if (has_bit) {
+    gulong offset = 0;
+    guint8 *nextext;
+    guint extlen;
+
+    if (bits != ((0x100 << 4) | (appbits & 0x0f)))
+      return FALSE;
+
+    offset = get_twobytes_header_end_offset (pdata, wordlen);
+
+    nextext = pdata + offset;
+
+    offset = nextext - GST_BUFFER_DATA (buffer);
+
+    /* Don't add extra header if there isn't enough space */
+    if (GST_BUFFER_SIZE (buffer) < offset + size + 2)
+      return FALSE;
+
+    nextext[0] = id;
+    nextext[1] = size;
+    memcpy (nextext + 2, data, size);
+
+    extlen = nextext - pdata + size + 2;
+    if (extlen % 4) {
+      wordlen = extlen / 4 + 1;
+      memset (nextext + size + 2, 0, 4 - extlen % 4);
+    } else {
+      wordlen = extlen / 4;
+    }
+
+    gst_rtp_buffer_set_extension_data (buffer, (0x100 << 4) | (appbits & 0x0F),
+        wordlen);
+  } else {
+    wordlen = (size + 1) / 4 + (((size + 1) % 4) ? 1 : 0);
+
+    gst_rtp_buffer_set_extension_data (buffer, (0x100 << 4) | (appbits & 0x0F),
+        wordlen);
+
+    gst_rtp_buffer_get_extension_data (buffer, &bits,
+        (gpointer) & pdata, &wordlen);
+
+    pdata[0] = id;
+    pdata[1] = size;
+    memcpy (pdata + 2, data, size);
+    if ((size + 2) % 4)
+      memset (pdata + size + 2, 0, 4 - ((size + 2) % 4));
+  }
+
+  return TRUE;
+}
+
+/**
+ * gst_rtp_buffer_list_get_extension_onebyte_header:
+ * @bufferlist: the bufferlist
+ * @group_idx: The index of the group in the #GstBufferList
+ * @id: The ID of the header extension to be read (between 1 and 14).
+ * @nth: Read the nth extension packet with the requested ID
+ * @data: location for data
+ * @size: the size of the data in bytes
+ *
+ * Parses RFC 5285 style header extensions with a one byte header. It will
+ * return the nth extension with the requested id.
+ *
+ * Returns: TRUE if @buffer had the requested header extension
+ *
+ * Since: 0.10.31
+ */
+
+gboolean
+gst_rtp_buffer_list_get_extension_onebyte_header (GstBufferList * bufferlist,
+    guint group_idx, guint8 id, guint nth, gpointer * data, guint * size)
+{
+  GstBuffer *buffer;
+
+  buffer = gst_buffer_list_get (bufferlist, group_idx, 0);
+
+  if (!buffer)
+    return FALSE;
+
+  return gst_rtp_buffer_get_extension_onebyte_header (buffer, id, nth, data,
+      size);
+}
+
+
+/**
+ * gst_rtp_buffer_list_get_extension_twobytes_header:
+ * @bufferlist: the bufferlist
+ * @group_idx: The index of the group in the #GstBufferList
+ * @appbits: Application specific bits
+ * @id: The ID of the header extension to be read (between 1 and 14).
+ * @nth: Read the nth extension packet with the requested ID
+ * @data: location for data
+ * @size: the size of the data in bytes
+ *
+ * Parses RFC 5285 style header extensions with a two bytes header. It will
+ * return the nth extension with the requested id.
+ *
+ * Returns: TRUE if @buffer had the requested header extension
+ *
+ * Since: 0.10.31
+ */
+
+gboolean
+gst_rtp_buffer_list_get_extension_twobytes_header (GstBufferList * bufferlist,
+    guint group_idx, guint8 * appbits, guint8 id, guint nth,
+    gpointer * data, guint * size)
+{
+  GstBuffer *buffer;
+
+  buffer = gst_buffer_list_get (bufferlist, group_idx, 0);
+
+  if (!buffer)
+    return FALSE;
+
+  return gst_rtp_buffer_get_extension_twobytes_header (buffer, appbits, id,
+      nth, data, size);
+}
+
+/**
+ * gst_rtp_buffer_list_add_extension_onebyte_header:
+ * @it: a #GstBufferListIterator pointing right after the #GstBuffer where
+ * the header extension should be added
+ * @id: The ID of the header extension (between 1 and 14).
+ * @data: location for data
+ * @size: the size of the data in bytes
+ *
+ * Adds a RFC 5285 header extension with a one byte header to the end of the
+ * RTP header. If there is already a RFC 5285 header extension with a one byte
+ * header, the new extension will be appended.
+ * It will not work if there is already a header extension that does not follow
+ * the mecanism described in RFC 5285 or if there is a header extension with
+ * a two bytes header as described in RFC 5285. In that case, use
+ * gst_rtp_buffer_list_add_extension_twobytes_header()
+ *
+ * This function will not modify the data section of the RTP buffer, only
+ * the header.
+ *
+ * Returns: %TRUE if header extension could be added
+ *
+ * Since: 0.10.31
+ */
+
+gboolean
+gst_rtp_buffer_list_add_extension_onebyte_header (GstBufferListIterator * it,
+    guint8 id, gpointer data, guint size)
+{
+  GstBuffer *buffer;
+  guint16 bits;
+  guint8 *pdata;
+  guint wordlen;
+  gboolean retval;
+  guint endoffset = 0;
+
+  g_return_val_if_fail (gst_buffer_list_iterator_n_buffers (it) == 1, FALSE);
+  g_return_val_if_fail (id > 0 && id < 15, FALSE);
+  g_return_val_if_fail (size >= 1 && size <= 16, FALSE);
+
+  buffer = gst_buffer_list_iterator_steal (it);
+
+  if (GST_RTP_HEADER_EXTENSION (GST_BUFFER_DATA (buffer))) {
+    gst_rtp_buffer_get_extension_data (buffer, &bits, (gpointer) & pdata,
+        &wordlen);
+
+    if (bits != 0xBEDE)
+      return FALSE;
+
+    endoffset = get_onebyte_header_end_offset (pdata, wordlen);
+    if (endoffset == 0)
+      return FALSE;
+    endoffset += pdata - GST_BUFFER_DATA (buffer);
+  } else {
+    endoffset = GST_BUFFER_SIZE (buffer) + 4;
+  }
+
+  if (endoffset + size + 1 > GST_BUFFER_SIZE (buffer)) {
+    guint newsize;
+    GstBuffer *newbuffer;
+
+    newsize = endoffset + size + 1;
+    if (newsize % 4)
+      newsize += 4 - (newsize % 4);
+    newbuffer = gst_buffer_new_and_alloc (newsize);
+    memcpy (GST_BUFFER_DATA (newbuffer), GST_BUFFER_DATA (buffer),
+        GST_BUFFER_SIZE (buffer));
+    gst_buffer_copy_metadata (newbuffer, buffer, GST_BUFFER_COPY_ALL);
+    gst_buffer_unref (buffer);
+    buffer = newbuffer;
+  } else {
+    buffer = gst_buffer_make_writable (buffer);
+  }
+
+  retval = gst_rtp_buffer_add_extension_onebyte_header (buffer, id, data, size);
+
+  gst_buffer_list_iterator_take (it, buffer);
+
+  return retval;
+}
+
+/**
+ * gst_rtp_buffer_list_add_extension_twobytes_header:
+ * @it: a #GstBufferListIterator pointing right after the #GstBuffer where
+ * the header extension should be added
+ * @appbits: Application specific bits
+ * @id: The ID of the header extension
+ * @data: location for data
+ * @size: the size of the data in bytes
+ *
+ * Adds a RFC 5285 header extension with a two bytes header to the end of the
+ * RTP header. If there is already a RFC 5285 header extension with a two bytes
+ * header, the new extension will be appended.
+ * It will not work if there is already a header extension that does not follow
+ * the mecanism described in RFC 5285 or if there is a header extension with
+ * a one byte header as described in RFC 5285. In that case, use
+ * gst_rtp_buffer_add_extension_onebyte_header()
+ *
+ * This function will not modify the data section of the RTP buffer, only
+ * the header.
+ *
+ * Returns: %TRUE if header extension could be added
+ *
+ * Since: 0.10.31
+ */
+
+gboolean
+gst_rtp_buffer_list_add_extension_twobytes_header (GstBufferListIterator * it,
+    guint8 appbits, guint8 id, gpointer data, guint size)
+{
+  GstBuffer *buffer;
+  guint16 bits;
+  guint8 *pdata;
+  guint wordlen;
+  gboolean retval;
+  guint endoffset;
+
+  g_return_val_if_fail ((appbits & 0xF0) == 0, FALSE);
+  g_return_val_if_fail (size < 256, FALSE);
+  g_return_val_if_fail (gst_buffer_list_iterator_n_buffers (it) == 1, FALSE);
+
+  buffer = gst_buffer_list_iterator_steal (it);
+
+  if (GST_RTP_HEADER_EXTENSION (GST_BUFFER_DATA (buffer))) {
+    gst_rtp_buffer_get_extension_data (buffer, &bits, (gpointer) & pdata,
+        &wordlen);
+
+    if (bits != ((0x100 << 4) | (appbits & 0x0f)))
+      return FALSE;
+
+    endoffset = get_twobytes_header_end_offset (pdata, wordlen);
+    if (endoffset == 0)
+      return FALSE;
+    endoffset += pdata - GST_BUFFER_DATA (buffer);
+  } else {
+    endoffset = GST_BUFFER_SIZE (buffer) + 4;
+  }
+
+  if (endoffset + size + 2 > GST_BUFFER_SIZE (buffer)) {
+    guint newsize;
+    GstBuffer *newbuffer;
+
+    newsize = endoffset + size + 2;
+    if (newsize % 4)
+      newsize += 4 - newsize % 4;
+    newbuffer = gst_buffer_new_and_alloc (newsize);
+    memcpy (GST_BUFFER_DATA (newbuffer), GST_BUFFER_DATA (buffer),
+        GST_BUFFER_SIZE (buffer));
+    gst_buffer_copy_metadata (newbuffer, buffer, GST_BUFFER_COPY_ALL);
+    gst_buffer_unref (buffer);
+    buffer = newbuffer;
+  } else {
+    buffer = gst_buffer_make_writable (buffer);
+  }
+
+  retval = gst_rtp_buffer_add_extension_twobytes_header (buffer, appbits, id,
+      data, size);
+
+  gst_buffer_list_iterator_take (it, buffer);
+
+  return retval;
+}
+
+/**
+ * gst_rtp_buffer_list_from_buffer:
+ * @buffer: a #GstBuffer containing a RTP packet
+ *
+ * Splits a #GstBuffer into a #GstBufferList containing separate
+ * buffers for the header and data sections.
+ *
+ * Returns: a #GstBufferList
+ */
+
+GstBufferList *
+gst_rtp_buffer_list_from_buffer (GstBuffer * buffer)
+{
+  GstBufferList *bufferlist;
+  GstBuffer *sub;
+  GstBufferListIterator *it;
+  guint8 *payload;
+
+  bufferlist = gst_buffer_list_new ();
+
+  it = gst_buffer_list_iterate (bufferlist);
+  gst_buffer_list_iterator_add_group (it);
+
+  payload = gst_rtp_buffer_get_payload (buffer);
+  sub = gst_buffer_create_sub (buffer, 0, payload - GST_BUFFER_DATA (buffer));
+  gst_buffer_list_iterator_add (it, sub);
+
+  sub = gst_rtp_buffer_get_payload_buffer (buffer);
+  gst_buffer_list_iterator_add (it, sub);
+
+  gst_buffer_list_iterator_free (it);
+
+  return bufferlist;
+}
