@@ -27,38 +27,27 @@
 
 #include "vs_scanline.h"
 
-#include <liboil/liboil.h>
+#include "gstvideoscaleorc.h"
+#include <gst/gst.h>
+
+#include <string.h>
 
 /* greyscale, i.e., single componenet */
 
 void
 vs_scanline_downsample_Y (uint8_t * dest, uint8_t * src, int n)
 {
-  int i;
-
-  for (i = 0; i < n; i++) {
-    dest[i] = (src[i * 2] + src[i * 2 + 1]) / 2;
-  }
+  orc_downsample_u8 (dest, src, n);
 }
 
 void
 vs_scanline_resample_nearest_Y (uint8_t * dest, uint8_t * src, int src_width,
     int n, int *accumulator, int increment)
 {
-  int acc = *accumulator;
-  int i;
-  int j;
-  int x;
+  gst_videoscale_orc_resample_nearest_u8 (dest, src,
+      *accumulator, increment, n);
 
-  for (i = 0; i < n; i++) {
-    j = acc >> 16;
-    x = acc & 0xffff;
-    dest[i] = (x < 32768 || j + 1 >= src_width) ? src[j] : src[j + 1];
-
-    acc += increment;
-  }
-
-  *accumulator = acc;
+  *accumulator += n * increment;
 }
 
 #include <glib.h>
@@ -66,24 +55,10 @@ void
 vs_scanline_resample_linear_Y (uint8_t * dest, uint8_t * src, int src_width,
     int n, int *accumulator, int increment)
 {
-  int acc = *accumulator;
-  int i;
-  int j;
-  int x;
+  gst_videoscale_orc_resample_bilinear_u8 (dest, src,
+      *accumulator, increment, n);
 
-  for (i = 0; i < n; i++) {
-    j = acc >> 16;
-    x = acc & 0xffff;
-
-    if (j + 1 < src_width)
-      dest[i] = (src[j] * (65536 - x) + src[j + 1] * x) >> 16;
-    else
-      dest[i] = src[j];
-
-    acc += increment;
-  }
-
-  *accumulator = acc;
+  *accumulator += n * increment;
 }
 
 void
@@ -92,18 +67,17 @@ vs_scanline_merge_linear_Y (uint8_t * dest, uint8_t * src1, uint8_t * src2,
 {
   uint32_t value = x >> 8;
 
-  oil_merge_linear_u8 (dest, src1, src2, &value, n);
+  if (value == 0) {
+    memcpy (dest, src1, n);
+  } else {
+    orc_merge_linear_u8 (dest, src1, src2, value, n);
+  }
 }
 
 void
 vs_scanline_downsample_Y16 (uint8_t * dest, uint8_t * src, int n)
 {
-  int i;
-  uint16_t *d = (uint16_t *) dest, *s = (uint16_t *) src;
-
-  for (i = 0; i < n; i++) {
-    d[i] = (s[i * 2] + s[i * 2 + 1]) / 2;
-  }
+  orc_downsample_u16 ((uint16_t *) dest, (uint16_t *) src, n);
 }
 
 void
@@ -127,7 +101,6 @@ vs_scanline_resample_nearest_Y16 (uint8_t * dest, uint8_t * src, int src_width,
   *accumulator = acc;
 }
 
-#include <glib.h>
 void
 vs_scanline_resample_linear_Y16 (uint8_t * dest, uint8_t * src, int src_width,
     int n, int *accumulator, int increment)
@@ -157,12 +130,14 @@ void
 vs_scanline_merge_linear_Y16 (uint8_t * dest, uint8_t * src1, uint8_t * src2,
     int n, int x)
 {
-  int i;
-  uint16_t *d = (uint16_t *) dest, *s1 = (uint16_t *) src1, *s2 =
-      (uint16_t *) src2;
+  uint16_t *d = (uint16_t *) dest;
+  const uint16_t *s1 = (const uint16_t *) src1;
+  const uint16_t *s2 = (const uint16_t *) src2;
 
-  for (i = 0; i < n; i++) {
-    d[i] = (s1[i] * (65536 - x) + s2[i] * x) >> 16;
+  if (x == 0) {
+    memcpy (d, s1, n * 2);
+  } else {
+    orc_merge_linear_u16 (d, s1, s2, 65536 - x, x, n);
   }
 }
 
@@ -171,81 +146,27 @@ vs_scanline_merge_linear_Y16 (uint8_t * dest, uint8_t * src1, uint8_t * src2,
 void
 vs_scanline_downsample_RGBA (uint8_t * dest, uint8_t * src, int n)
 {
-  int i;
-
-  for (i = 0; i < n; i++) {
-    dest[i * 4 + 0] = (src[i * 8 + 0] + src[i * 8 + 4]) / 2;
-    dest[i * 4 + 1] = (src[i * 8 + 1] + src[i * 8 + 5]) / 2;
-    dest[i * 4 + 2] = (src[i * 8 + 2] + src[i * 8 + 6]) / 2;
-    dest[i * 4 + 3] = (src[i * 8 + 3] + src[i * 8 + 7]) / 2;
-  }
+  gst_videoscale_orc_downsample_u32 (dest, src, n);
 }
 
 void
 vs_scanline_resample_nearest_RGBA (uint8_t * dest, uint8_t * src, int src_width,
     int n, int *accumulator, int increment)
 {
-  int acc = *accumulator;
-  int i;
-  int j;
-  int x;
+  gst_videoscale_orc_resample_nearest_u32 (dest, src,
+      *accumulator, increment, n);
 
-  for (i = 0; i < n; i++) {
-    j = acc >> 16;
-    x = acc & 0xffff;
-
-    if (j + 1 < src_width) {
-      dest[i * 4 + 0] = (x < 32768) ? src[j * 4 + 0] : src[j * 4 + 4];
-      dest[i * 4 + 1] = (x < 32768) ? src[j * 4 + 1] : src[j * 4 + 5];
-      dest[i * 4 + 2] = (x < 32768) ? src[j * 4 + 2] : src[j * 4 + 6];
-      dest[i * 4 + 3] = (x < 32768) ? src[j * 4 + 3] : src[j * 4 + 7];
-    } else {
-      dest[i * 4 + 0] = src[j * 4 + 0];
-      dest[i * 4 + 1] = src[j * 4 + 1];
-      dest[i * 4 + 2] = src[j * 4 + 2];
-      dest[i * 4 + 3] = src[j * 4 + 3];
-    }
-
-    acc += increment;
-  }
-
-  *accumulator = acc;
+  *accumulator += n * increment;
 }
 
-#include <stdio.h>
 void
 vs_scanline_resample_linear_RGBA (uint8_t * dest, uint8_t * src, int src_width,
     int n, int *accumulator, int increment)
 {
-  uint32_t vals[2];
+  gst_videoscale_orc_resample_bilinear_u32 (dest, src,
+      *accumulator, increment, n);
 
-  vals[0] = *accumulator;
-  vals[1] = increment;
-
-  if (src_width % 2 == 0) {
-    oil_resample_linear_argb ((uint32_t *) dest, (uint32_t *) src, n, vals);
-  } else if (src_width > 1) {
-    if (n > 1)
-      oil_resample_linear_argb ((uint32_t *) dest, (uint32_t *) src, n - 1,
-          vals);
-    dest[4 * (n - 1) + 0] = src[(vals[0] >> 16) + 0];
-    dest[4 * (n - 1) + 1] = src[(vals[0] >> 16) + 1];
-    dest[4 * (n - 1) + 2] = src[(vals[0] >> 16) + 2];
-    dest[4 * (n - 1) + 3] = src[(vals[0] >> 16) + 3];
-    vals[0] += increment;
-  } else {
-    int i;
-
-    for (i = 0; i < n; i++) {
-      dest[4 * i + 0] = src[0];
-      dest[4 * i + 1] = src[1];
-      dest[4 * i + 2] = src[2];
-      dest[4 * i + 3] = src[3];
-      vals[0] += increment;
-    }
-  }
-
-  *accumulator = vals[0];
+  *accumulator += n * increment;
 }
 
 void
@@ -254,8 +175,11 @@ vs_scanline_merge_linear_RGBA (uint8_t * dest, uint8_t * src1, uint8_t * src2,
 {
   uint32_t value = x >> 8;
 
-  oil_merge_linear_argb ((uint32_t *) dest, (uint32_t *) src1,
-      (uint32_t *) src2, &value, n);
+  if (value == 0) {
+    memcpy (dest, src1, n * 4);
+  } else {
+    orc_merge_linear_u8 (dest, src1, src2, value, n * 4);
+  }
 }
 
 
@@ -336,7 +260,11 @@ vs_scanline_merge_linear_RGB (uint8_t * dest, uint8_t * src1, uint8_t * src2,
 {
   uint32_t value = x >> 8;
 
-  oil_merge_linear_u8 (dest, src1, src2, &value, n * 3);
+  if (value == 0) {
+    memcpy (dest, src1, n * 3);
+  } else {
+    orc_merge_linear_u8 (dest, src1, src2, value, n * 3);
+  }
 }
 
 
@@ -348,14 +276,7 @@ vs_scanline_merge_linear_RGB (uint8_t * dest, uint8_t * src1, uint8_t * src2,
 void
 vs_scanline_downsample_YUYV (uint8_t * dest, uint8_t * src, int n)
 {
-  int i;
-
-  for (i = 0; i < n; i++) {
-    dest[i * 4 + 0] = (src[i * 8 + 0] + src[i * 8 + 2]) / 2;
-    dest[i * 4 + 1] = (src[i * 8 + 1] + src[i * 8 + 5]) / 2;
-    dest[i * 4 + 2] = (src[i * 8 + 4] + src[i * 8 + 6]) / 2;
-    dest[i * 4 + 3] = (src[i * 8 + 3] + src[i * 8 + 7]) / 2;
-  }
+  gst_videoscale_orc_downsample_yuyv (dest, src, n);
 }
 
 void
@@ -458,21 +379,13 @@ void
 vs_scanline_merge_linear_YUYV (uint8_t * dest, uint8_t * src1, uint8_t * src2,
     int n, int x)
 {
-  int i;
   int quads = (n + 1) / 2;
+  uint32_t value = x >> 8;
 
-  for (i = 0; i < quads; i++) {
-    dest[i * 4 + 0] =
-        (src1[i * 4 + 0] * (65536 - x) + src2[i * 4 + 0] * x) >> 16;
-    dest[i * 4 + 1] =
-        (src1[i * 4 + 1] * (65536 - x) + src2[i * 4 + 1] * x) >> 16;
-
-    if (2 * i + 1 < n) {
-      dest[i * 4 + 2] =
-          (src1[i * 4 + 2] * (65536 - x) + src2[i * 4 + 2] * x) >> 16;
-      dest[i * 4 + 3] =
-          (src1[i * 4 + 3] * (65536 - x) + src2[i * 4 + 3] * x) >> 16;
-    }
+  if (value == 0) {
+    memcpy (dest, src1, quads * 4);
+  } else {
+    orc_merge_linear_u8 (dest, src1, src2, value, quads * 4);
   }
 }
 
@@ -595,21 +508,13 @@ void
 vs_scanline_merge_linear_UYVY (uint8_t * dest, uint8_t * src1,
     uint8_t * src2, int n, int x)
 {
-  int i;
   int quads = (n + 1) / 2;
+  uint32_t value = x >> 8;
 
-  for (i = 0; i < quads; i++) {
-    dest[i * 4 + 0] =
-        (src1[i * 4 + 0] * (65536 - x) + src2[i * 4 + 0] * x) >> 16;
-    dest[i * 4 + 1] =
-        (src1[i * 4 + 1] * (65536 - x) + src2[i * 4 + 1] * x) >> 16;
-
-    if (2 * i + 1 < n) {
-      dest[i * 4 + 2] =
-          (src1[i * 4 + 2] * (65536 - x) + src2[i * 4 + 2] * x) >> 16;
-      dest[i * 4 + 3] =
-          (src1[i * 4 + 3] * (65536 - x) + src2[i * 4 + 3] * x) >> 16;
-    }
+  if (value == 0) {
+    memcpy (dest, src1, quads * 4);
+  } else {
+    orc_merge_linear_u8 (dest, src1, src2, value, quads * 4);
   }
 }
 
