@@ -64,7 +64,7 @@ typedef struct _GstXvImageBufferClass GstXvImageBufferClass;
 typedef struct _GstXvImageSink GstXvImageSink;
 typedef struct _GstXvImageSinkClass GstXvImageSinkClass;
 
-/**
+/*
  * GstXContext:
  * @disp: the X11 Display of this context
  * @screen: the default Screen of Display @disp
@@ -128,9 +128,11 @@ struct _GstXContext {
   /* Optimisation storage for buffer_alloc return */
   GstCaps *last_caps;
   gint last_format;
+  gint last_width;
+  gint last_height;
 };
 
-/**
+/*
  * GstXWindow:
  * @win: the Window ID of this X11 window
  * @width: the width in pixels of Window @win
@@ -248,6 +250,7 @@ struct _GstXvImageSink {
   GValue *par;
 
   GMutex *pool_lock;
+  gboolean pool_invalid;
   GSList *image_pool;
 
   gboolean synchronous;
@@ -266,11 +269,27 @@ struct _GstXvImageSink {
   /* size of incoming video, used as the size for XvImage */
   guint video_width, video_height;
 
+  /* display sizes, used for clipping the image */
+  gint disp_x, disp_y;
+  gint disp_width, disp_height;
+
   /* port attributes */
   gboolean autopaint_colorkey;
   gint colorkey;
   
   gboolean draw_borders;
+  
+  /* port features */
+  gboolean have_autopaint_colorkey;
+  gboolean have_colorkey;
+  gboolean have_double_buffer;
+  
+  /* stream metadata */
+  gchar *media_title;
+
+  /* target video rectagle */
+  GstVideoRectangle render_rect;
+  gboolean have_render_rect;
 };
 
 struct _GstXvImageSinkClass {
